#! /usr/local/bin/perl -w

use GDBM_File;
use Getopt::Long;
use Storable qw(freeze thaw);

$dbmfile = "gismu.gdbm";
$gismufile = "gismu";
unless ( GetOptions(
		    "dbfile=s" => \$dbmfile,
		    "gismu=s" => \$gismufile,
		   ))
{
    die "bad option\n";
}

print "Building database $dbmfile from gismu in $gismufile..\n";
tie %gismu, "GDBM_File", $dbmfile, &GDBM_WRCREAT, 0644
  or die "couldn't create database: $!";
open(GISMU, "<$gismufile") or die "couldn't open gismu: $!";
my $dummy = <GISMU>;		# skip first line, it's a header
my $count = 0;
my $skipped = 0;
while (<GISMU>) {
    # annoyingly, the gismu file is build with fixed-length fields
    chomp;
    # $code at column 160? is "lesson and sublesson" from an early draft 
    # textbook
    # $number at column 157? (<=4 digits) is a frequency of occurence
    unless (/^[a-z]{5} /) {
	$skip++; # skip non-gismu
	#print "skipping: ",substr($_,0,5),"\n";
	next;
    }
    my($gismu, $rafsi1, $rafsi2, $rafsi3, $english, $metaphor, 
       $place_structure, $code, $number, $refs) =
	 # 0   6      19     40     61     158   160   168
	 /^(.{5}) (.{3}) (.{3}) (.{4}) (.{21})(.{21})(.{97})(.{2})(.{4})(.*)$/;
    die "$.: '$_'" unless defined($number);
    $gismu =~ s/\s+$//; $gismu =~ s/^\s+//;
    $rafsi1 =~ s/\s+$//; $rafsi1 =~ s/^\s+//;
    $rafsi2 =~ s/\s+$//; $rafsi2 =~ s/^\s+//;
    $rafsi3 =~ s/\s+$//; $rafsi3 =~ s/^\s+//;
    $english =~ s/\s+$//; $english =~ s/^\s+//;
    $metaphor =~ s/\s+$//; $metaphor =~ s/^\s+//;
    $place_structure =~ s/\s+$//; $place_structure =~ s/^\s+//;
    $code =~ s/\s+$//; $code =~ s/^\s+//;
    $number =~ s/\s+$//; $number =~ s/^\s+//;
    $refs =~ s/\s+$//; $refs =~ s/^\s+//;
    
    my $bad = 0;
    #print "$.: $gismu\n";
    #next unless $gismu =~ /^[a-z]{5}$/; # remove non-gismu
    #print("$.: $gismu: not gismu\n"),$bad++
    #  unless length($gismu) == 5 and $gismu !~ /\'/;
    print("$.: rafsi1 $rafsi1"),$bad++ if $rafsi1 =~ / /;
    print("$.: rafsi2 $rafsi2"),$bad++ if $rafsi2 =~ / /;
    print("$.: rafsi3 $rafsi3"),$bad++ if $rafsi3 =~ / /;
    #die("$.: english $english") if $english =~ / /;
    print("$.: code $code"),$bad++ if $code =~ / /;
    print("$.: number $number"),$bad++ if $number =~ / /;
    if ($bad) {
	print "bad line $. '$_'\n";
	print "gismu: '$gismu'\n";
	print "rafsi1: '$rafsi1'\n";
	print "rafsi2: '$rafsi2'\n";
	print "rafsi3: '$rafsi3'\n";
	print "english: '$english'\n";
	print "metaphor: '$metaphor'\n";
	print "place_structure: '$place_structure'\n";
	print "code: '$code'\n";
	print "number: '$number'\n";
	print "refs: '$refs'\n";
	die;
    }
    
    #print "$refs\n" if $refs;
    $gismu{$gismu} = freeze( {
			      gismu => $gismu,
			      rafsi1 => $rafsi1,
			      rafsi2 => $rafsi2,
			      rafsi3 => $rafsi3,
			      english => $english,
			      metaphor => $metaphor,
			      place_structure => $place_structure,
			      code => $code,
			      number => $number,
			      refs => $refs,
			     } );
    $count++;
}
untie %gismu;
close(GISMU);

print "$count gismu included, $skip non-gismu skipped\n";
